<?php
namespace MASCOTCOREINSOLAN\CPT\Works;

use MASCOTCOREINSOLAN\Lib;

/**
 * Singleton class
 * class CPT_Works
 * @package MASCOTCOREINSOLAN\CPT\Works;
 */
final class CPT_Works implements Lib\Mascot_Core_Insolan_Interface_PostType {
	
	/**
	 * @var string
	 */
	public  $ptKey;
	public 	$ptKeyRewriteBase;
	public  $ptTaxKey;
	public  $ptTaxKeyRewriteBase;
	private $ptMenuIcon;
	private $ptSingularName;
	private $ptPluralName;

	/**
	 * Call this method to get singleton
	 *
	 * @return CPT_Works
	 */
	public static function Instance() {
		static $inst = null;
		if ($inst === null) {
			$inst = new CPT_Works();
		}
		return $inst;
	}
	
	/**
	 * Private ctor so nobody else can instance it
	 *
	 */
	private function __construct() {
		$this->ptSingularName = esc_html__( 'Works Item', 'mascot-core-insolan-wp' );
		$this->ptPluralName = esc_html__( 'Works', 'mascot-core-insolan-wp' );
		$this->ptKey = 'works';
		$this->ptKeyRewriteBase = $this->ptKey;
		$this->ptTaxKey = 'works_category';
		$this->ptTaxKeyRewriteBase = str_replace( '_', '-', $this->ptTaxKey );
		$this->ptMenuIcon = 'dashicons-portfolio';
		add_filter( 'manage_edit-'.$this->ptKey.'_columns', array($this, 'customColumnsSettings') ) ;
		add_filter( 'manage_'.$this->ptKey.'_posts_custom_column', array($this, 'customColumnsContent') ) ;

		add_filter( 'rwmb_meta_boxes', array($this, 'regMetaBoxes') ) ;

		add_filter('manage_edit-'.$this->ptTaxKey.'_columns' , array($this, 'my_custom_taxonomy_columns'));
		add_filter( 'manage_'.$this->ptTaxKey.'_custom_column', array($this, 'my_custom_taxonomy_columns_content'), 10, 3 );
	}

	/**
	 * @return string
	 */
	public function getPTKey() {
		return $this->ptKey;
	}

	/**
	 * @return string
	 */
	public function getPTTaxKey() {
		return $this->ptTaxKey;
	}

	/**
	 * registers custom post type & Tax
	 */
	public function register() {
		if ( class_exists( 'ReduxFramework' ) ) {
			if( ! mascot_core_insolan_get_redux_option( 'cpt-settings-works-enable', true ) ) {
				return;
			}
		}
		
		$this->ptPluralName = mascot_core_insolan_get_redux_option( 'cpt-settings-works-label', esc_html__( 'Works', 'mascot-core-insolan-wp' ) );
		$this->ptMenuIcon = mascot_core_insolan_get_redux_option( 'cpt-settings-works-admin-dashicon', $this->ptMenuIcon );
		$this->ptKeyRewriteBase = mascot_core_insolan_get_redux_option( 'cpt-settings-works-slug', $this->ptKeyRewriteBase );
		$this->ptTaxKeyRewriteBase = mascot_core_insolan_get_redux_option( 'cpt-settings-works-cat-slug', $this->ptKeyRewriteBase );
		
		$this->registerCustomPostType();
		$this->registerCustomTax();
	}
	
	/**
	 * Regsiters custom post type
	 */
	private function registerCustomPostType() {

		$labels = array(
			'name'					=> $this->ptPluralName,
			'singular_name'			=> $this->ptPluralName . ' ' . esc_html__( 'Item', 'mascot-core-insolan-wp' ),
			'menu_name'				=> $this->ptPluralName,
			'name_admin_bar'		=> $this->ptPluralName,
			'archives'				=> esc_html__( 'Item Archives', 'mascot-core-insolan-wp' ),
			'attributes'			=> esc_html__( 'Item Attributes', 'mascot-core-insolan-wp' ),
			'parent_item_colon'		=> esc_html__( 'Parent Item:', 'mascot-core-insolan-wp' ),
			'all_items'				=> esc_html__( 'All Items', 'mascot-core-insolan-wp' ),
			'add_new_item'			=> esc_html__( 'Add New Item', 'mascot-core-insolan-wp' ),
			'add_new'				=> esc_html__( 'Add New', 'mascot-core-insolan-wp' ),
			'new_item'				=> esc_html__( 'New Item', 'mascot-core-insolan-wp' ),
			'edit_item'				=> esc_html__( 'Edit Item', 'mascot-core-insolan-wp' ),
			'update_item'			=> esc_html__( 'Update Item', 'mascot-core-insolan-wp' ),
			'view_item'				=> esc_html__( 'View Item', 'mascot-core-insolan-wp' ),
			'view_items'			=> esc_html__( 'View Items', 'mascot-core-insolan-wp' ),
			'search_items'			=> esc_html__( 'Search Item', 'mascot-core-insolan-wp' ),
			'not_found'				=> esc_html__( 'Not found', 'mascot-core-insolan-wp' ),
			'not_found_in_trash'	=> esc_html__( 'Not found in Trash', 'mascot-core-insolan-wp' ),
			'featured_image'		=> esc_html__( 'Featured Image', 'mascot-core-insolan-wp' ),
			'set_featured_image'	=> esc_html__( 'Set featured image', 'mascot-core-insolan-wp' ),
			'remove_featured_image'	=> esc_html__( 'Remove featured image', 'mascot-core-insolan-wp' ),
			'use_featured_image'	=> esc_html__( 'Use as featured image', 'mascot-core-insolan-wp' ),
			'insert_into_item'		=> esc_html__( 'Insert into item', 'mascot-core-insolan-wp' ),
			'uploaded_to_this_item'	=> esc_html__( 'Uploaded to this item', 'mascot-core-insolan-wp' ),
			'items_list'			=> esc_html__( 'Items list', 'mascot-core-insolan-wp' ),
			'items_list_navigation'	=> esc_html__( 'Items list navigation', 'mascot-core-insolan-wp' ),
			'filter_items_list'		=> esc_html__( 'Filter items list', 'mascot-core-insolan-wp' ),
		);

		$args = array(
			'label'						=> $this->ptSingularName,
			'description'				=> esc_html__( 'Post Type Description', 'mascot-core-insolan-wp' ),
			'labels'					=> $labels,
			'supports'					=> array( 'title', 'thumbnail', 'editor', 'excerpt', 'page-attributes' ),
			'taxonomies'				=> array( $this->ptTaxKey ),
			'hierarchical'				=> true,
			'public'					=> true,
			'show_ui'					=> true,
			'show_in_menu'				=> true,
			'menu_position'				=> 30,
			'menu_icon'					=> $this->ptMenuIcon,
			'show_in_admin_bar'			=> true,
			'show_in_nav_menus'			=> true,
			'can_export'				=> true,
			'has_archive'				=> true,
			'exclude_from_search'		=> false,
			'publicly_queryable'		=> true,
			'capability_type'			=> 'page',
			'rewrite'					=> array( 'slug' => $this->ptKeyRewriteBase, 'with_front' => false ),
		);
		register_post_type( $this->ptKey, $args );

	}
	
	/**
	 * Regsiters custom Taxonomy
	 */
	private function registerCustomTax() {

		$labels = array(
			'name'						=> _x( 'Works Categories', 'Taxonomy General Name', 'mascot-core-insolan-wp' ),
			'singular_name'				=> _x( 'Work Category', 'Taxonomy Singular Name', 'mascot-core-insolan-wp' ),
			'menu_name'					=> $this->ptSingularName . esc_html__( ' Categories', 'mascot-core-insolan-wp' ),
			'all_items'					=> esc_html__( 'All ', 'mascot-core-insolan-wp' ) . $this->ptSingularName . esc_html__( ' Categories', 'mascot-core-insolan-wp' ),
			'parent_item'				=> esc_html__( 'Parent Item', 'mascot-core-insolan-wp' ),
			'parent_item_colon'			=> esc_html__( 'Parent Item:', 'mascot-core-insolan-wp' ),
			'new_item_name'				=> esc_html__( 'New ', 'mascot-core-insolan-wp' ) . $this->ptSingularName . esc_html__( ' Category', 'mascot-core-insolan-wp' ),
			'add_new_item'				=> esc_html__( 'Add ', 'mascot-core-insolan-wp' ) . $this->ptSingularName . esc_html__( ' Category', 'mascot-core-insolan-wp' ),
			'edit_item'					=> esc_html__( 'Edit ', 'mascot-core-insolan-wp' ) . $this->ptSingularName . esc_html__( ' Category', 'mascot-core-insolan-wp' ),
			'update_item'				=> esc_html__( 'Update ', 'mascot-core-insolan-wp' ) . $this->ptSingularName . esc_html__( ' Category', 'mascot-core-insolan-wp' ),
			'view_item'					=> esc_html__( 'View ', 'mascot-core-insolan-wp' ) . $this->ptSingularName . esc_html__( ' Category', 'mascot-core-insolan-wp' ),
			'separate_items_with_commas'=> esc_html__( 'Separate items with commas', 'mascot-core-insolan-wp' ),
			'add_or_remove_items'		=> esc_html__( 'Add or remove items', 'mascot-core-insolan-wp' ),
			'choose_from_most_used'		=> esc_html__( 'Choose from the most used', 'mascot-core-insolan-wp' ),
			'popular_items'				=> esc_html__( 'Popular Items', 'mascot-core-insolan-wp' ),
			'search_items'				=> esc_html__( 'Search Items', 'mascot-core-insolan-wp' ),
			'not_found'					=> esc_html__( 'Not Found', 'mascot-core-insolan-wp' ),
			'no_terms'					=> esc_html__( 'No items', 'mascot-core-insolan-wp' ),
			'items_list'				=> esc_html__( 'Items list', 'mascot-core-insolan-wp' ),
			'items_list_navigation'		=> esc_html__( 'Items list navigation', 'mascot-core-insolan-wp' ),
		);
		$args = array(
			'labels'					=> $labels,
			'hierarchical'				=> true,
			'public'					=> true,
			'show_ui'					=> true,
			'show_admin_column'			=> true,
			'show_in_nav_menus'			=> true,
			'show_tagcloud'				=> true,
			'rewrite'					=> array( 'slug' => $this->ptTaxKeyRewriteBase, 'with_front' => false ),
		);
		register_taxonomy( $this->ptTaxKey, array( $this->ptKey ), $args );
	}
	
	/**
	 * Custom Columns Settings
	 */
	public function customColumnsSettings( $columns ) {

		$columns = array(
			'cb'			=> '<input type="checkbox" />',
			'title'			=> esc_html__( 'Title', 'mascot-core-insolan-wp' ),
			'thumbnail'		=> esc_html__( 'Thumbnail', 'mascot-core-insolan-wp' ),
			'category'		=> esc_html__( 'Category', 'mascot-core-insolan-wp' ),
			'date'			=> esc_html__( 'Date', 'mascot-core-insolan-wp' ),
		);
		return $columns;
	}

	/**
	 * Custom Columns Content
	 */
	public function customColumnsContent( $columns ) {
		global $post;
		switch( $columns ) {
			case 'category' :
				$post_terms = array();
				$taxonomy 	= $this->ptTaxKey;
				$post_type 	= get_post_type( $post->ID );
				$terms 		= get_the_terms( $post->ID, $taxonomy );

				if ( ! empty( $terms ) ) {
					foreach ( $terms as $term ) {
						$post_terms[] = "<a href='edit.php?post_type={$post_type}&{$taxonomy}={$term->slug}'> " . esc_html( sanitize_term_field( 'name', $term->name, $term->term_id, $taxonomy, 'edit' ) ) . "</a>";
					}
					echo join( ', ', $post_terms );
				}
				else echo '<i>No categories.</i>';
			break;

			case 'thumbnail' :
				echo get_the_post_thumbnail( $post->ID, array( 64, 64 ) );
			break;

			default :
			break;
		}
	}

	/**
	 * Registers Meta Boxes
	 */
	public function regMetaBoxes( $meta_boxes ) {
		$meta_boxes[] = array(
			'title'	  => esc_html__( 'Additional Meta Box', 'mascot-core-insolan-wp' ),
			'taxonomies'   => $this->ptTaxKey,
			'fields'		=> array(
				array(
					'id'	=> 'featured_image',
					'name'		=> esc_html__( 'Featured Image', 'mascot-core-insolan-wp' ),
					'type'	=> 'image_advanced',
					'max_file_uploads' => 1,
				),
			),
		);
		return $meta_boxes;
	}
	public function my_custom_taxonomy_columns( $columns )
	{
		$columns['cat_thumbnail'] = esc_html__('Thumbnail');

		return $columns;
	}
	public function my_custom_taxonomy_columns_content( $content, $column_name, $term_id )
	{
		if ( 'cat_thumbnail' == $column_name ) {

			$thumb = rwmb_meta( 'featured_image', array( 'object_type' => 'term', 'limit' => 1, 'size' => 'thumbnail' ), $term_id );
			$thumb = reset( $thumb );

			if ( !_empty( $thumb ) ) {
				$content = '<img src="'.esc_url( $thumb['url'] ).'" alt="">';
			}
		}
		return $content;
	}
}